defmodule Smol do
  use GenServer
  require Logger

  @crlf "\r\n"
  @schemes ["gemini", "spartan"]

  def main(args \\ [])

  def main(nil) do
    IO.puts("At least one argument is required")
  end

  def main(args) do
    args
    |> parse_args()
    |> response()
  
    receive do
      {:all_done_boss} -> System.halt(0)
    end
  end
  
  def parse_args(args) do
    args
    |> OptionParser.parse(strict: [request: :string])
  end

  def response({opts, word, invalid} = args) do
    if opts[:request], do: request(opts[:request])
  end

  #defp error_response(opt) do 
  #  IO.puts("No option #{opt}")
  #end

  def request(url) do
    uri = URI.parse(url)
    if Enum.any?(@schemes, fn u -> u === uri.scheme end) do
      process = Smol.start_link("")
      #IO.inspect(process)
      GenServer.call(__MODULE__, {:connect, url}, :infinity)
    else
      IO.puts("Invalid url")
    end
  end

  @impl true
  def init(state) do
    {:ok, state}
  end

  def start_link(initial_state) do
    GenServer.start_link(__MODULE__, initial_state, name: __MODULE__)
  end

  @impl true
  def handle_call({:connect, url}, _from, state) do
    {:reply, connect(url), state}
  end

  @impl true
  def handle_info({:ssl, _sslsocket, content}, state) do
    #Logger.info("Received message with content: #{inspect(content)}" )
    {:noreply, state <> parse_response(to_string(content))}
  end

  @impl true
  def handle_info({:ssl_closed, _sslsocket}, state) do
    {:stop, :normal, state}
  end

  @impl true
  def terminate(reason, state) do
    IO.puts(state)
    :normal
  end

  def set_opts do
    [verify: :verify_none, active: :true]
  end

  def connect(url) do
    uri = URI.parse(url)
    :ssl.start()
    port = 1965

    {:ok, sslsocket} = :ssl.connect(String.to_charlist(uri.host), port, set_opts())
    crlf_url = url <> @crlf

    status = :ssl.send(sslsocket, String.to_charlist(crlf_url))
    unless status === :ok do
      IO.puts("Error sending request")
    end
  end

  def parse_response(content) do
    String.replace(content, "20 text/gemini\r\n", "")
    |> to_string
  end

  def process_gemini(content) do
    String.replace(content, "20 text/gemini\r\n", "")
    |> to_string
  end

  def process_spartan(_content) do
    IO.puts("Not implemented!")
  end
end
